#
#  TCL script used to configure and dial an Intel 14.4EX modem.
#
# Assumes that the zero'th argument to the top level script is the phone number
# which needs to be dialed.
#

proc dial-intel-144ex { DIALER number cfg } {
    
    global Config
    
    # set default timeout for expect/rexpect commands
    set timeout 20
    set how TONE
    
    syslog LOG_INFO "Start of Intel 14.4EX dialing script, dialing $number"
    
    $DIALER chardelay 35
    # flush any pending command
    $DIALER xmit {\r}
    $DIALER sleep 1
    
    # get modems attention
    set timeout 2
    foreach i {once} {
        $DIALER xmit {AT\r}
        $DIALER expect "*OK\r*" {break} timeout {}
        $DIALER xmit {AT\r}
        $DIALER expect "*OK\r*" {break} timeout {}
        $DIALER xmit {AT\r}
        $DIALER expect "*OK\r*" {break} timeout {}
        error "Could not get the modem's attention"
    }
    set timeout 5
    
    # turn off command echo
    $DIALER xmit {ATE0\r}
    $DIALER expect timeout {error "waiting for OK"} "*OK*"
    
    # configure modem with proper parameters 
    
    # V1        verbose codes sent as words
    # &Q0       standard result codes (not hayes 2400)
    # X4        Hayes result code set with NO DIAL DONE and BUSY
    # &E1       V.42 auto-reliable mode				either/
    # &E2       V.42 reliable mode				   or
    # &E4       CTS modem initiated flow control
    # &E7       Xon/Xoff passed through
    # &E8       Enq/Ack pacing off
    # &E11      Normal Mode flow control on
    # &E13      Pacing on (RTS or XON/XOFF depending on &E4 or &E5)
    # &E15      Data compression enabled
    # $MB14400  modem baud rate
    # $SB38400  serial port baud rate
    # M0        monitor speaker always off			either/
    # M1        Monitor speaker on until carrier detected	  or
    # $BA0      baud adjust off (speed conversion on)
    # &C1       carrier detect acts normally			either/
    # &C2       carrier normally asserted, excpet at disconnect   or
    # #L3       No detection phase -- go directly to LAPM
    # S24=20    drop time in 50ms increments
    # S2=128    turn off escape character
    #
    # (Don't have to escape $ inside of {} quoted string)
    #
    set setupstr { V1 &Q0 X4
	M1 &C1 S24=20 S2=128 }
    set setupstr { Q0V1 X4 &C1 &D2 S0=0 S2=128  S7=30 S11=55 }

	
    foreach s $setupstr {
	$DIALER xmit "AT$s\r"
	$DIALER expect timeout {error "waiting for OK for parm $s"} \
	    "{*ERROR\r*}" {syslog LOG_WARNING "Modem returned ERROR setting $s"} \
	    "{*OK*}" 
    }


    # do these one at a time, to get the escape-character (\) handled right.
    $DIALER xmit {AT\\G0\r}
    $DIALER expect timeout {error "waiting for OK for parm \\G0"} \
            "{*ERROR\r*}" {syslog LOG_WARNING "Modem returned ERROR for \\G0"} \
            "{*OK*}" 
    $DIALER xmit {AT\\J0\r}
    $DIALER expect timeout {error "waiting for OK for parm \\J0"} \
            "{*ERROR\r*}" {syslog LOG_WARNING "Modem returned ERROR for \\J0"} \
            "{*OK*}"
    $DIALER xmit {AT\\N4\r}
    $DIALER expect timeout {error "waiting for OK for parm \\N4"} \
            "{*ERROR\r*}" {syslog LOG_WARNING "Modem returned ERROR for \\N4"} \
            "{*OK*}"
    $DIALER xmit {AT\\Q3\r}
    $DIALER expect timeout {error "waiting for OK for parm \\Q3"} \
            "{*ERROR\r*}" {syslog LOG_WARNING "Modem returned ERROR for \\Q3"} \
            "{*OK*}"
    $DIALER xmit {AT\\V2\r}
    $DIALER expect timeout {error "waiting for OK for parm \\V2"} \
            "{*ERROR\r*}" {syslog LOG_WARNING "Modem returned ERROR for \\V2"} \
            "{*OK*}"


    # allow override of dialing type
    if {[info exists Config($cfg:DIALTYPE)]} {
	set how $Config($cfg:DIALTYPE)
    }
    
    # dial the phone
    if { [string compare [string tolower $how] tone] &&
	 [string compare [string tolower $how] dtmf] } {
	     syslog LOG_INFO "Dialing (pulse) $number ..."
	     $DIALER xmit "ATDP$number\r"
	 } else {
	     syslog LOG_INFO "Dialing (DTMF) $number ..."
	     $DIALER xmit "ATDT$number\r"
	 }
    
    # wait for connect message
    set timeout 40
    $DIALER expect \
	timeout {error "Timeout waiting for modem to connect to $number"} \
	"{*CONNECT 14400/REL-LAPM-COMP*}" {syslog LOG_INFO "Connected 14400bps LAPM"} \
	"{*CONNECT 14400 LAPM COMPRESSED\r*}" {syslog LOG_INFO "Connected 14400bps LAPM"} \
	"{*CONNECT 14400 RELIABLE COMPRESSED\r*}" {syslog LOG_INFO "Connected 14400bps"} \
	"{*CONNECT 14400\r*}" {syslog LOG_INFO "Connected 14400bps NO MNP5!"} \
	"{*CONNECT 9600 LAPM COMPRESSED\r*}" {syslog LOG_INFO "Connected 9600bps LAPM"} \
	"{*CONNECT 9600 RELIABLE COMPRESSED\r*}" {syslog LOG_INFO "Connected 9600bps"} \
	"{*CONNECT 9600\r*}" {syslog LOG_INFO "Connected 9600bps NO MNP5!"} \
	"{*CONNECT\r*}" {syslog LOG_INFO "Connected (some random way)"} \
	"{*BUSY\r*}" {error "Remote modem busy"}
    
    # set expect timeout back to something "reasonable"
    set timeout 20
}
