# Copyright 1994 by Garance Alistair Drosehn.
# Somewhat based on scripts written by TransSys, Inc.
# All rights reserved.

proc xyplex-password { DIALER username password } {
    $DIALER xmit "$username\r"
#    $DIALER expect "*assword:*"  {$DIALER xmit "$password\r"} \
#	timeout	   {error "waiting for password prompt"}
#    $DIALER xmit "$password\r"
}

#  Used to put the Xyplex terminal server at RPI into SLIP mode. 
#
# This version will use the username variable, but ignore the
# password (since the RPI server does not ask for a password).
# The username and password to be used can still be stored in
# a seperate file.  The name of the file is specified in the
# Config array.
#
# This file should contain (at least) two TCL set commands to
# set the variables 'username' and 'password'.  Its likely
# that you don't want these files generally readable for
# security purposes, so a check is made for you, and the
# script will fail if the file is readable.
#
# Alternatively, the members of the keyed list "username" and
# "password" can be used to specify the username and password
# to be passed to the terminal server.

proc login-rpi-xyplex { DIALER cfg } {
    global Config
    global ConsoleMsgPfx
    
    syslog LOG_INFO "Begin RPI xyplex login"
    
    set username ""
    set password ""
    # the following is a control-k
    set eraseinp ""
    
    if {[info exists Config($cfg:SECRETFILE)] && 
	[file exists $Config($cfg:SECRETFILE)]} {
	    
	    if {[file readable $Config($cfg:SECRETFILE)]} {
		source $Config($cfg:SECRETFILE)
	    }
	    
	    file stat $Config($cfg:SECRETFILE) stat
	    if {[expr $stat(mode)&04]} {
		error "File with password, $Config($cfg:SECRETFILE), is readable by 'other'!"
	    }
	} else {
	    if {[info exists Config($cfg:USERNAME)]} {
		set username $Config($cfg:USERNAME)
	    }
	    if {[info exists Config($cfg:PASSWORD)]} {
		set password $Config($cfg:PASSWORD)
	    }
	}
    
    #
    # set parity of transmitted data
    #
    $DIALER parity ZERO
    #
    #  For auto-baud nonsense.. sometimes its necessary to poke at it a couple
    #  of times before it figures out what speed the modem is at.
    #
    $DIALER sleep 2
    set timeout 8

    foreach i {once} {
	$DIALER xmit {\r}
	#
	#  look for xyplex request for username, or prompt for menu item.
	#
	$DIALER expect "{*Main Menu*}" {break} timeout {} \
	    "*sername>*" {xyplex-password $DIALER $username $password
		break}
	$DIALER xmit {\r}
	
	$DIALER expect "{*Main Menu*}" {break} timeout {} \
	    "*sername>*" {xyplex-password $DIALER $username $password
		break}
	$DIALER xmit {\r}
		
	error "Couldn't autobaud xyplex: no prompt from xyplex terminal server"
    }
    
    $DIALER sleep 3
    set timeout 15
    
    #
    #  Run down the menu structure to find out our IP address.
    foreach i {once} {
	$DIALER expect "{*number of selection*}" {break} timeout {}

	error "Didn't get initial main screen"
    }
    $DIALER xmit "$eraseinp"
    $DIALER xmit "13\r"
    $DIALER expect "{*Menu 6:*}" {} timeout {error "no menu 6"}
    foreach i {once} {
	$DIALER expect "{*number of selection*}" {break} timeout {}

	error "Couldn't switch to List Alt Port Params screen"
    }
    $DIALER xmit "$eraseinp"
    $DIALER xmit "1\r"
    #
    #  Use regular-expression based expect command to match the
    #  address in the message that the Xyplex server tells us.
    $DIALER expect "{*DTR wait:*}" {} timeout {}
    foreach i {once} {
	$DIALER rexpect \
	    {SLIP Address: *([0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+)} { break } \
    }
    set originalRouteAddr $Config(pni:ADDRESS)
    set Config(pni:ADDRESS) $1.$2.$3.$4
    pni local-ip-address $Config(pni:ADDRESS)

    set originalRemoteAddr $Config(pni:REMOTEADDRESS)
    if { $4 < 30 } {
        set Config(pni:REMOTEADDRESS) 128.113.29.210
    } else {
	if { $4 < 50 } {
	    set Config(pni:REMOTEADDRESS) 128.113.29.211
	} else {
	    set Config(pni:REMOTEADDRESS) 128.113.29.212
	}
    }
    pni remote-ip-address $Config(pni:REMOTEADDRESS)

    # The following expects the config file set DEFAULT 0 and NOLOOPBACK 1:
    exec /usr/etc/route delete $originalRemoteAddr $originalRouteAddr
    exec /usr/etc/ifconfig $Config(pni:INTERFACE) inet $Config(pni:ADDRESS) $Config(pni:REMOTEADDRESS) up arp netmask $Config(pni:NETWORKMASK)
    # note that the ifconfig effectively does a:
  # exec /usr/etc/route add $Config(pni:REMOTEADDRESS) $Config(pni:ADDRESS) 0
    # add the loopback routing entry:
    exec /usr/etc/route add $Config(pni:ADDRESS) 127.0.0.1 0
    # add the default routing entry:
    exec /usr/etc/route add default $Config(pni:REMOTEADDRESS) 1

    syslog LOG_INFO "Xyplex server IP address is $Config(pni:REMOTEADDRESS)"
    syslog LOG_INFO "Xyplex port   IP address is $Config(pni:ADDRESS)"
    
    #
    #  get back to top-most (initial) screen, and send the
    #  command to put terminal server into SLIP mode
    #
    $DIALER xmit "$eraseinp"
    $DIALER xmit "\r"
    foreach i {once} {
	$DIALER expect "{*number of selection*}" {break} timeout {}
        $DIALER xmit "$eraseinp"
	$DIALER xmit {\r}
	
	$DIALER expect "{*number of selection*}" {break} timeout {}

	error "Got addr, but couldn't return to List Alt Port Params screen"
    }
    $DIALER xmit "$eraseinp"
    $DIALER xmit "t\r"

    set mode SLIP
    if {[info exists Config(slip:MODE)]} {
	set mode $Config(slip:MODE)
    }
    case $mode in {
	{SLIP slip default}	{$DIALER xmit "3"}
	{CSLIP cslip}		{$DIALER xmit "4"}
#	{PPP ppp}		{$DIALER xmit "5"}
    }
    $DIALER sleep 1
    $DIALER expect "*" {} timeout {}
    $DIALER xmit "\r"
    
    syslog LOG_DEBUG "Entering $mode mode"
    exec   /bin/echo "$ConsoleMsgPfx myIP=$Config(pni:ADDRESS) xyplexIP=$Config(pni:REMOTEADDRESS)" > /dev/console

    #
    # wait a bit for server to switch to SLIP mode
    #
    $DIALER sleep 1
    #
    # that's all
    #
    return "Connected"
}
